//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package main

import (
	"encoding/json"
	"net/http"

	"github.com/gorilla/mux"
)

type webAPIv1Endpoint struct {
	Methods []string `json:"allowed-methods"`
}

var (
	webAPIv1Endpoints = make(map[string]*webAPIv1Endpoint)
)

func webAPIv1_Index(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Content-Type", "application/json")
	json.NewEncoder(w).Encode(webAPIv1Endpoints)
}

func webAPIv1_Groups_List(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "listing groups not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_PlaylistsOfGroup_List(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "listing playlists of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_PlaylistsOfGroup_Create(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "creating playlists in group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_PlaylistsOfGroup_Read(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "reading playlist of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_PlaylistsOfGroup_Update(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "updateing playlist of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_PlaylistsOfGroup_Delete(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "deleting playlist from group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_FilesOfGroup_List(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "listing files of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_FilesOfGroup_Create(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "create files in group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_FilesOfGroup_Read(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "reading file of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_FilesOfGroup_Update(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "updateing file of group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1_FilesOfGroup_Delete(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "deleting file from group not yet implemented", http.StatusNotImplemented)
}

func webAPIv1__methodNotAllowed(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "method not allowed", http.StatusMethodNotAllowed)
}

func webAPIv1InstallHandler(r *mux.Router) {
	// Groups
	r.HandleFunc("/groups", webAPIv1_Groups_List).Methods("GET")
	r.HandleFunc("/groups", webAPIv1__methodNotAllowed)

	// Playlists
	r.HandleFunc("/groups/{group-id}/playlists", webAPIv1_PlaylistsOfGroup_List).Methods("GET")
	r.HandleFunc("/groups/{group-id}/playlists", webAPIv1_PlaylistsOfGroup_Create).Methods("POST")
	r.HandleFunc("/groups/{group-id}/playlists", webAPIv1__methodNotAllowed)
	r.HandleFunc("/groups/{group-id}/playlists/{playlist-id}", webAPIv1_PlaylistsOfGroup_Read).Methods("GET")
	r.HandleFunc("/groups/{group-id}/playlists/{playlist-id}", webAPIv1_PlaylistsOfGroup_Update).Methods("PUT")
	r.HandleFunc("/groups/{group-id}/playlists/{playlist-id}", webAPIv1_PlaylistsOfGroup_Delete).Methods("DELETE")
	r.HandleFunc("/groups/{group-id}/playlists/{playlist-id}", webAPIv1__methodNotAllowed)

	// Files
	r.HandleFunc("/groups/{group-id}/files", webAPIv1_FilesOfGroup_List).Methods("GET")
	r.HandleFunc("/groups/{group-id}/files", webAPIv1_FilesOfGroup_Create).Methods("POST")
	r.HandleFunc("/groups/{group-id}/files", webAPIv1__methodNotAllowed)

	r.HandleFunc("/groups/{group-id}/files/{file-id}", webAPIv1_FilesOfGroup_Read).Methods("GET")
	r.HandleFunc("/groups/{group-id}/files/{file-id}", webAPIv1_FilesOfGroup_Update).Methods("PUT")
	r.HandleFunc("/groups/{group-id}/files/{file-id}", webAPIv1_FilesOfGroup_Delete).Methods("DELETE")
	r.HandleFunc("/groups/{group-id}/files/{file-id}", webAPIv1__methodNotAllowed)

	// Index
	r.HandleFunc("/", webAPIv1_Index).Methods("GET")
	r.HandleFunc("/", webAPIv1__methodNotAllowed)
	r.Walk(func(route *mux.Route, router *mux.Router, ancestors []*mux.Route) error {
		p, err := route.GetPathTemplate()
		if err != nil {
			return nil
		}

		m, err := route.GetMethods()
		if err != nil || len(m) == 0 {
			return err
		}

		if e, exists := webAPIv1Endpoints[p]; exists {
			e.Methods = append(e.Methods, m...)
		} else {
			webAPIv1Endpoints[p] = &webAPIv1Endpoint{m}
		}
		return nil
	})
}
